
# title: "Section 2: Models for Binary Outcomes: Part 2"
# subtitle: ICPSR CDA
# author: Tamara van der Does\ 2017-07-05

################################################################################

# 1.a Set Up
 
## Load libraries
library(car)
library(foreign)
library(psych)
 
# 1.b Load the Data
## Download the data saved previously
dataclean <-as.data.frame(read.dta("cda_scireview3_tvdd.dta", convert.factor=TRUE))

# 1.c Re-estimate your Binary Logit Model 
Results should match your results in BRM-Part 1

mod.logit <- glm(faculty~ fellow+phd+mcit3+mnas, data=dataclean, family="binomial")
summary(mod.logit)

# 2 Predicted Probabilities for Observed Data

## Use "names" to see the results stored in model
names(mod.logit)

## The predicted probabilities are stored as "fitted.values"
summary(mod.logit$fitted.values)

## Table with summary statistics for predicted probabilities
predprob <- cbind(mean(mod.logit$fitted.values), sd(mod.logit$fitted.values), length(mod.logit$fitted.values), min(mod.logit$fitted.values), max(mod.logit$fitted.values))
colnames(predprob) <- c("mean", "standard dev", "N", "Min", "Max")
rownames(predprob) <- c("prlogit")
predprob
 

## Histogram of predicted probabilities

## classic version
hist(mod.logit$fitted.values, main="Dotplot of Predicted Probabilities")
 
## GGplot version
library(easyGgplot2)
 
ggplot(dataclean, aes(x = mod.logit$fitted.values)) + geom_dotplot(method="histodot", binwidth = 0.012, dotsize=0.75, color="blue") + scale_y_continuous(NULL, breaks = NULL) + labs(title="Dotplot of Predicted Probabilities") + xlab("Logit: Predicted Probabilities")
 
## Save plot
ggsave("icpsrcda02-binary-fig1.png")

# 3-5 Discrete Change with Confidence Interval

## For binary variable
  
## Create sub-sample
s.fel <- with(dataclean, data.frame(fellow=factor(1:2, levels=1:2, labels=levels(dataclean$fellow)), phd=mean(phd), mcit3=mean(mcit3), mnas="0_No"))

## Verify subsample created correctly
s.fel

## Apply our logit model to the subsample
pred.fel <- predict(mod.logit, s.fel, type="response", se.fit=TRUE) 

## Difference between fellows and non fellows
diff <- pred.fel$fit[2] - pred.fel$fit[1]

## Calculate the confidence intervals
CI.U <- diff + (1.96 * sqrt(pred.fel$se.fit[1]^2+pred.fel$se.fit[2]^2))
CI.L <- diff - (1.96 * sqrt(pred.fel$se.fit[1]^2+pred.fel$se.fit[2]^2))

## Discrete change and confidence interval
cbind(diff, CI.L,CI.U)
 
## For continuous variable

## Save the values for C
mcit.centL <- mean(dataclean$mcit3)-(0.5*sd(dataclean$mcit3)) 
mcit.centU <- mean(dataclean$mcit3)+(0.5*sd(dataclean$mcit3))

## Create the subsets with other variables at their means
s.U <- with(dataclean, data.frame(fellow="0_No", mcit3=mcit.centU, phd=mean(phd), mnas="0_No"))
s.L <- with(dataclean, data.frame(fellow="0_No", mcit3=mcit.centL, phd=mean(phd), mnas="0_No"))
s.U
s.L 

## Apply our logit model to the subsample
pred.U <- predict(mod.logit, s.U, type="response", se.fit=TRUE) 
pred.L <- predict(mod.logit, s.L, type="response", se.fit=TRUE)

diff <- pred.U$fit - pred.L$fit

## Calculate the confidence intervals
CI.U <- diff + (1.96 * sqrt(pred.U$se.fit^2+pred.L$se.fit^2))
CI.L <- diff - (1.96 * sqrt(pred.U$se.fit^2+pred.L$se.fit^2))

## Discrete change and confidence interval
cbind(diff, CI.L, CI.U)
 
# 6 Plot predicted probabilities
To plot predicted probabilities by B over C, first create a subsample with C as a sequence, B as factor and the rest held at values of interest. Then attach the predicted probabilities for that data. Calculate and add the confidence interval to the data. Finally use the dataset to create a plot. 

## Create subsample
s.plot <- with(dataclean, data.frame(mcit3= rep(seq(from = 0, to = 130, length.out = 100), 2), phd = mean(phd), mnas="0_No", fellow = factor(rep(1:2, each = 100), labels=levels(dataclean$fellow))))

## Create data with subsample and predicted probabilities
s.plot.data <- cbind(s.plot, predict(mod.logit, newdata = s.plot, type="link", se=TRUE))

## Add confidence interval 
s.plot.data <- within(s.plot.data, {
PredictedProb <- plogis(fit)
LL <- plogis(fit - (1.96 * se.fit))
UL <- plogis(fit + (1.96 * se.fit))
})

## Look at data created 
head(s.plot.data)

## Plot of Predicted Probabilities using ggplot
ggplot(s.plot.data, aes(x = mcit3, y = PredictedProb)) + geom_ribbon(aes(ymin = LL, ymax = UL, fill = fellow), alpha = .2) + geom_line(aes(colour = fellow), size=1) + labs(title="Predicted Probability of Having a Faculty Position", x="Mentor's # of Citations", y="Pr(Fellow)") + theme(legend.title=element_blank())
 
## Save plot
ggsave("icpsrcda02-binary-fig2.png")
 
