
# title: "Section 3: Testing and Assessing Fit"
# subtitle: ICPSR CDA
# author: Tamara van der Does\ 2017-07-05

################################################################################

# 1.a Set up
 
## Load libraries
library(car)
library(foreign)
library(psych)
library(easyGgplot2)
 

# 1.b Load the Data

data <-as.data.frame(read.dta('https://shawnana79.github.io/data/cda_scireview3.dta', convert.factor=TRUE))
 
# 1.c Examine Data, keep variables, drop missing and verify.  

## Describe the data 
head(data)

## Select variables of interest
myvars <- c("faculty", "female", "fellow", "phd", "mcit3", "mnas")
datasub <- data[myvars]

## Make sure the classs of variables are correct
datasub$faculty <- as.factor(datasub$faculty)
datasub$fellow <- as.factor(datasub$fellow)
datasub$female <- as.factor(datasub$female)
datasub$mnas <- as.factor(datasub$mnas)
datasub$mcit3 <- as.numeric(datasub$mcit3)
datasub$phd <- as.numeric(datasub$phd)

## Only keep non missing variables
dataclean <- na.omit(datasub)

## Verify data is clean
sum(is.na(dataclean)) 
dataclean[!complete.cases(dataclean),]
summary(dataclean)
describe(dataclean)
 

# 2. Computing a Z-test

mod.log <- glm(faculty~female+fellow+phd+mcit3+mnas, data=dataclean, family=binomial)
summary(mod.log)
 
# 3.a Single coefficient Wald test

## Package for Wald tests
library(aod)
 
## Run the test on 2nd coefficient of model
wald.test(b=coef(mod.log), Sigma= vcov(mod.log), Terms=2)

# 4. Single Coefficient LR Test
   
## First, run a new model without your variable
mod2.log <- glm(faculty~fellow+phd+mcit3+mnas, data=dataclean, family=binomial)

## Then, test the two models using the anova command
anova(mod2.log, mod.log, test="Chisq")
 

# 5. Multiple Coefficient Wald Test 

## Run a Wald test on coefficients 5 and 6
wald.test(b=coef(mod.log), Sigma= vcov(mod.log), Terms=5:6)
 
# 6. Multiple Coefficient LR test

## Create estimates of model without the two varialbes
mod3.log <- glm(faculty~female+fellow+phd, data=dataclean, family=binomial)

## Test against the full model
anova(mod3.log, mod.log, test="Chisq")
 
# 7. Wald Test All Coefficients are Zero
  
wald.test(b=coef(mod.log), Sigma= vcov(mod.log), Terms=2:6)
 
# 8. LR test All Coefficients are Zero
  
## Use LR test code
LRtest <- mod.log$null.deviance - mod.log$deviance
LRtest

## calculate p-value of  test
pvalue <-dchisq(LRtest, mod.log$df.null - mod.log$df.residual)
pvalue
 
# 9. More Complicated Wald Tests

## Coefficients equal to each other
 
## Create a vector matching the estimates 
vect1 <- cbind(0, 0,0,0,1,-1)

## Conduct Wald test with vector
wald.test(b=coef(mod.log), Sigma= vcov(mod.log), L=vect1)
 

## Effect of female is twice the effect of mcit3.
vect2 <- cbind(0, -1,0,0,2,0)
wald.test(b=coef(mod.log), Sigma= vcov(mod.log), L=vect2)
 

# 10.a Compare BIC and AIC Statistics Across Non-nested Models

## Run and save each models
mod.loga <-glm(faculty~female+fellow, data=dataclean, family=binomial)
mod.logb <- glm(faculty~mcit3, data=dataclean, family=binomial)
mod.logc <- glm(faculty~phd+mnas, data=dataclean, family=binomial)
mod.logd <- glm(faculty~phd+mnas+female+fellow, data=dataclean, family=binomial)

## Save AIC and BIC for each model
mods.aic <- c(AIC(mod.loga),AIC(mod.logb),AIC(mod.logc),AIC(mod.logd))
mods.bic <- c(BIC(mod.loga),BIC(mod.logb),BIC(mod.logc),BIC(mod.logd))

## Present test statistics
mods.tab <- cbind(mods.aic, mods.bic)
colnames(mods.tab) <- c("AIC", "BIC")
rownames(mods.tab) <- c("m1", "m2", "m3", "m4")
mods.tab
 

# 11. Compute and List Residuals 
 
## Model and predicted probabilities 
predprob <- mod.log$fitted.values
data2 <- cbind(dataclean,predprob)

## Residuals 
res <- residuals(mod.log, "pearson")/(sqrt(1 - hatvalues(mod.log)))

## add to data
data3 <-cbind(data2,res)

## Sort and present 10 highest and lowest
sort1.data3 <- data3[order(res) , ]
sort1.data3[1:10, ]

sort2.data3 <-data3[order(-res),]
sort2.data3[1:10,]
 

