
# title: "Section 4: Models for Ordinal Outcomes"
# subtitle: ICPSR CDA
# author: Tamara van der Does\ 2017-07-05

################################################################################

# 1.a Set Up

## Load libraries
library(car)
library(foreign)
library(psych)
library(easyGgplot2)
library(aod)
 

# 1.b Load the Data
  
data <-as.data.frame(read.dta('https://shawnana79.github.io/data/cda_scireview3.dta', convert.factor=TRUE))
 
# 1.c Examine Data, Keep Variables, Drop Missing and Verify.  

head(data)
myvars <- c("jobprst", "pub1", "phd", "female")
datasub <- data[myvars]
datasub$jobprst <- as.factor(datasub$jobprst)
datasub$female <- as.factor(datasub$female)
datasub$phd <- as.numeric(datasub$phd)
datasub$pub1 <- as.numeric(datasub$pub1)
dataclean <- na.omit(datasub)
sum(is.na(dataclean)) 
dataclean[!complete.cases(dataclean),]
 
# 2 Produce Table including Distribution of Output Variable 
  
describe(dataclean)

## Example on how to do it manually
coltitles <- cbind("Name", "Mean", "Std Dev", "Min", "Max", "Description")
firstcol <- rbind("jobprst", "pub1", "female", "phd")
seccol <- rbind("", "2.32", "0.35", "3.18")
thirdcol <- rbind("", "2.58", "", "1.01")
fourcol <- rbind(1, min(dataclean$pub1), 0, min(dataclean$phd))
fifthcol <- rbind(4, max(dataclean$pub1), 1, "4.66")
sixcol <- rbind("1=Adeq, 2=Good, 3=Strong, 4=Dist", "Publications", "1=Female, 0=Male", "Phd program prestige")
allcol <- cbind(firstcol, seccol, thirdcol, fourcol, fifthcol, sixcol)
allcol

# 3. Estimate the Ordered Logit
  
## Load library to run polr
library(MASS)
 
## Present model
mod.olog <- polr(jobprst~pub1 + female+phd, data=dataclean, Hess=T)
summary(mod.olog)

# 4-6 Odds Ratios

## Save odds ratios, CI, stx
bHat<- coef(mod.olog)
ci <- exp(cbind(confint.default(mod.olog)))
sdX <- c(sd(dataclean$pub1), NA, sd(dataclean$phd))

## Calculate precentage changes
perc.bHat <- (exp(bHat)-1)*100
perc.sdx <-  (exp(bHat*sdX)-1)*100

## Put everything together and round to 4 digits
facOdds <- round(cbind(bHat, ci, exp(bHat), exp(bHat*sdX), sdX, perc.bHat, perc.sdx), digits=3)
colnames(facOdds) <- c("b", "LL", "UL", "e^b", "e^(b*sd)", "SD of X", "%", "%StdX")
facOdds
 
# 7. Compute Discrete Change for C and B

## C: Pub1
  
## Set up the values of the changes for C (+1 centered)
pub.centL <- mean(dataclean$pub1)-0.5 
pub.centU <- mean(dataclean$pub1)+0.5

## Create subsets and check
s.U <- with(dataclean, data.frame(female="0_No", phd=mean(phd), pub1=pub.centU))
s.U
s.L <- with(dataclean, data.frame(female="0_No", phd=mean(phd), pub1=pub.centL))
s.L 

## Apply our model to subsample
pred.U <- predict(mod.olog, s.U, type="probs", se.fit=TRUE) 
pred.L <- predict(mod.olog, s.L, type="probs", se.fit=TRUE)

## Discrete change
diff <- pred.U - pred.L
diff
 
## B: Female

## Subsample
s.fem <- with(dataclean, data.frame(pub1=mean(pub1), female=factor(1:2, levels=1:2, labels=levels(female)), phd=mean(phd)))

## Prediction
pred.fem <- predict(mod.olog, s.fem, type="probs", se.fit=TRUE) 

## Discrete change
diff <- pred.fem[2,]-pred.fem[1,]
diff

# 8. Graph Predcited Probabilities 

## Load necessary libraries
library(effects)
  
## Create a data frame of the data
effect1 <- effect("pub1", mod.olog, typical=mean)
effect1 <- data.frame(effect1)

## Plot Graph
ggplot(effect1, aes(pub1)) + geom_line(aes(y=prob.X1_Adeq, colour="Adequate")) + geom_line(aes(y=prob.X2_Good, colour="Good")) + geom_line(aes(y=prob.X3_Strong, colour="Strong")) + geom_line(aes(y=prob.X4_Dist, colour="Distinguished")) + scale_colour_hue(breaks=c("Adequate", "Good", "Strong", "Distinguished")) + labs(title="Predicted Probabilities of quality of job", x="Number of Publications", y="Predicted Probability", colour="Category") 

## Save Graph  
ggsave("icpsrcda04-ordinal-fig2.png")
   

## Create cummulative probabilities 
effect1$prob.X1X2<- effect1$prob.X1_Adeq+effect1$prob.X2_Good
effect1$prob.X1X2X3 <- effect1$prob.X1X2+effect1$prob.X3_Strong
effect1$prob.X1X2X3X4 <- effect1$prob.X1X2X3+effect1$prob.X4_Dist

## Plot
ggplot(effect1, aes(pub1)) + geom_line(aes(y=prob.X1_Adeq, colour="Adequate")) + geom_line(aes(y=prob.X1X2, colour="Good")) + geom_line(aes(y=prob.X1X2X3, colour="Strong")) + geom_line(aes(y=prob.X1X2X3X4, colour="Distinguished")) + scale_colour_hue(breaks=c("Adequate", "Good", "Strong", "Distinguished")) + geom_ribbon(data=effect1, aes(x=pub1, ymin=0, ymax=prob.X1_Adeq), fill="firebrick2", alpha="0.5") + geom_ribbon(data=effect1, aes(x=pub1, ymin=prob.X1_Adeq, ymax=prob.X1X2), fill="lightskyblue", alpha="0.5") + geom_ribbon(data=effect1, aes(x=pub1, ymin=prob.X1X2, ymax=prob.X1X2X3), fill="mediumpurple", alpha="0.5") + geom_ribbon(data=effect1, aes(x=pub1, ymin=prob.X1X2X3, ymax=prob.X1X2X3X4), fill="olivedrab3", alpha="0.5") + labs(title="Cumulative Predicted Probabilities of quality of job", x="Number of Publications", y="Predicted Probability", colour="Category") 

ggsave("icpsrcda04-ordinal-fig3.png")
 
# 14. Testing the Parallel Regression Assumption

## load libraries
library(ordinal)

## Create models with forced prallel assumption for each variable
ologit.parallel <- clm(jobprst~pub1 + female+phd, data=dataclean, link="logit")
ologit.test1 <- clm(jobprst~female+phd, nominal= ~pub1, data=dataclean, link="logit")
ologit.test2<- clm(jobprst~pub1+female, nominal= ~phd, data=dataclean, link="logit")
ologit.test3<- clm(jobprst~pub1+phd, nominal= ~female, data=dataclean, link="logit")

## Test these models with regular model using LR test
anova(ologit.parallel, ologit.test1)
anova(ologit.parallel, ologit.test2)
anova(ologit.parallel, ologit.test3)
 
