
# title: "Section 5: Models for Nominal Outcomes"
# subtitle: ICPSR CDA
# author: Tamara van der Does\ 2017-07-05

################################################################################

# 1.a Set Up

# Load libraries
library(car)
library(foreign)
library(psych)
library(easyGgplot2)
library(aod)
library(MASS)
library(effects)
library(ordinal)
 
# 1.b Load the Data
  
data <-as.data.frame(read.dta('https://shawnana79.github.io/data/cda_scireview3.dta', convert.factor=TRUE))
 
# 1.c Examine Data, Keep Variables, Drop Missing  

head(data)
myvars <- c("jobprst", "female", "mcit3", "pub1", "phd") 
datasub <- data[myvars]
datasub$jobprst <- as.factor(datasub$jobprst)
datasub$pub1 <- as.numeric(datasub$pub1)
datasub$phd <- as.numeric(datasub$phd)
datasub$female <- as.factor(datasub$female)
datasub$mcit3 <- as.numeric(datasub$mcit3)
dataclean <- na.omit(datasub)
sum(is.na(dataclean)) 
dataclean[!complete.cases(dataclean),]
 
#2. Verify Data is Clean using Descriptives
  
summary(dataclean)
describe(dataclean)
 
#4. Multinomial Logit
 
## load library for multinom
library(nnet)
 
## Chose reference category for data
dataclean$jobprst <- relevel(dataclean$jobprst, ref = "4_Dist")

## Run multinomial logit using "multinom"
mlogit.mod <- multinom(jobprst~pub1+female+phd+mcit3, data=dataclean, Hess=TRUE)
summary(mlogit.mod)

    
## Save coefficients, z-scores, and standard deviation
bHat<- coef(mlogit.mod)[,2:5]
z <- summary(mlogit.mod)$coefficients[,2:5]/summary(mlogit.mod)$standard.errors[,2:5]
sdX <- cbind(cbind(replicate(3,sd(dataclean$pub1))), cbind(replicate(3,NA)), cbind(replicate(3, sd(dataclean$phd))), cbind(replicate(3,sd(dataclean$mcit3))))

## Create tables for each covariate
facOdds.pub <- cbind(bHat[,1],exp(bHat[,1]), exp(bHat[,1]*sdX[,1]), z[,1], sdX[,1])
facOdds.fem <- cbind(bHat[,2],exp(bHat[,2]), exp(bHat[,2]*sdX[,2]),  z[,2], sdX[,2])
facOdds.phd <- cbind(bHat[,3],exp(bHat[,3]), exp(bHat[,3]*sdX[,3]),  z[,3], sdX[,3])
facOdds.mcit3 <- cbind(bHat[,4],exp(bHat[,4]), exp(bHat[,4]*sdX[,4]),  z[,4], sdX[,4])

## Name the columns 
colnames(facOdds.pub) <- c("b", "e^b", "e^(b*sd)", "z-score", "SD of X")
colnames(facOdds.fem) <- c("b", "e^b", "e^(b*sd)", "z-score", "SD of X")
colnames(facOdds.phd) <- c("b", "e^b", "e^(b*sd)", "z-score", "SD of X")
colnames(facOdds.mcit3) <- c("b", "e^b", "e^(b*sd)", "z-score", "SD of X")

## Present the factor changes in odds
facOdds.pub
facOdds.fem
facOdds.phd
facOdds.mcit3
 
# 5. Single Coefficient Wald and LR Test

## 2-tailed Z test 

z <- summary(mlogit.mod)$coefficients/summary(mlogit.mod)$standard.errors
chi2 <- z^2
chi2
p <- (1 - pnorm(abs(z), 0, 1))*2
p
 
## LR Test
Anova(mlogit.mod)

# 6. Plot Odds Ratios

## Get info on for all comparisons
dataclean$jobprst <- relevel(dataclean$jobprst, ref = "1_Adeq")
mlogit.mod.1 <- multinom(jobprst~pub1+female+phd+mcit3, data=dataclean, Hess=TRUE)
ci.1 <- confint(mlogit.mod.1, level=0.95)

dataclean$jobprst <- relevel(dataclean$jobprst, ref = "2_Good")
mlogit.mod.2 <- multinom(jobprst~pub1+female+phd+mcit3, data=dataclean, Hess=TRUE)
ci.2 <- confint(mlogit.mod.2, level=0.95)

dataclean$jobprst <- relevel(dataclean$jobprst, ref = "3_Strong")
mlogit.mod.3 <- multinom(jobprst~pub1+female+phd+mcit3, data=dataclean, Hess=TRUE)
ci.3 <- confint(mlogit.mod.3, level=0.95)

dataclean$jobprst <- relevel(dataclean$jobprst, ref = "4_Dist")
mlogit.mod.4 <- multinom(jobprst~pub1+female+phd+mcit3, data=dataclean, Hess=TRUE)
ci.4 <- confint(mlogit.mod.4, level=0.95)

## Look at conference intervals for all comparisons

## CI for OR compared to having Distinguished job
Dist.ci <- cbind(cbind(ci.4[3,1:2,1]), cbind(ci.4[3,1:2,2]), cbind(ci.4[3,1:2,3]))
colnames(Dist.ci) <- c("3_Strong", "2_Good", "1_Adeq")
Dist.ci

## CI for OR compared to having Strong job
Strong.ci <- cbind(cbind(ci.3[3,1:2,1]), cbind(ci.3[3,1:2,2]))
colnames(Strong.ci) <- c("2_Good", "1_Adeq")
Strong.ci

## CI for OR compared to having Good job
Good.ci <- cbind(cbind(ci.2[3,1:2,1]))
colnames(Good.ci) <- c("1_Adeq")
Good.ci

## Save odd ratios compared to distinguished
or.fem  <- exp(bHat[,2])
 
## Create groups to have line for significance
sig <- as.factor(rbind(1, 2, 3, 1, 2, 3,4, 4))

## Create dataframe matching groups with labels and OR
dataorplot <- data.frame(class=c("1_Adeq", "1_Adeq", "1_Adeq", "2_Good", "3_Strong", "4_Dist", "3_Strong", "4_Dist"), or=c(or.fem[1], or.fem[1], or.fem[1], or.fem[2], or.fem[3], 1, or.fem[3], 1))
dataorplot$sig <-sig

## Plot
plot <- ggplot(dataorplot, aes(x=class, y=or, ymax=15,group = sig, col=class)) +  geom_point() + geom_line(col="dimgrey") + geom_text(aes(label=class),hjust=0.5, vjust=-1) + labs(title="Odds Ratio Relative to Category 4_Dist")
plot + theme(axis.text.x=element_blank(), legend.position = "none")

# 8. Calculating Discrete Change

## For B
s.fem <- with(dataclean, data.frame(pub1=mean(pub1), female=factor(1:2, levels=1:2, labels=levels(dataclean$female)), phd=mean(phd), mcit3=mean(mcit3)))
head(s.fem)

fem.pred <- predict(mlogit.mod, s.fem, type="probs", se.fit=T)
diff.fem <- fem.pred[2,]-fem.pred[1,]
diff.fem

## For C 
mcit3.U <- mean(dataclean$mcit3)+(0.5*sd(dataclean$mcit3))
mcit3.L <- mean(dataclean$mcit3)-(0.5*sd(dataclean$mcit3))

s.mcitU <- with(dataclean, data.frame(pub1=mean(pub1), female="0_No", phd=mean(phd), mcit3=mcit3.U))
head(s.mcitU)
s.mcitL <- with(dataclean, data.frame(pub1=mean(pub1), female="0_No", phd=mean(phd), mcit3=mcit3.L))
head(s.mcitL)

mcitU.pred <- predict(mlogit.mod, s.mcitU, type="probs", se.fit=T)
mcitL.pred <- predict(mlogit.mod, s.mcitL, type="probs", se.fit=T)
diff.mcit <- mcitU.pred - mcitL.pred
diff.mcit
 

# 8. Plotting Discrete Change
  
## Save data
mnlpred1 <- as.numeric(diff)

## Add labels for the four outcomes
mnlpred1 <- as.data.frame(cbind(rbind("1_Adeq", "2_Good", "3_Strong", "4_Dist"), mnlpred1))

## Add column names and change as numeric
colnames(mnlpred1) <- c("category", "Discretechange")
mnlpred1$Discretechange <- as.numeric(mnlpred1$Discretechange)

## Plot
plot2 <- ggplot(mnlpred1, aes(x=category,y=Discretechange, ymax=4.1, col=category)) + geom_point() + geom_text(aes(label=category), hjust=0.2, vjust=-0.7) + labs(title="Discrete Change in Probabilities for Gender")
plot2 + theme(axis.text.x=element_blank(), legend.position = "none")
 
# 8. Calculating Discrete Change II 

## For B
s.fem2 <- with(dataclean, data.frame(pub1=4, female=factor(1:2, levels=1:2, labels=levels(dataclean$female)), phd=4, mcit3=mean(mcit3)))
head(s.fem2)

fem.pred2 <- predict(mlogit.mod, s.fem2, type="probs", se.fit=T)
diff.fem2 <- fem.pred2[2,]-fem.pred2[1,]
diff.fem2

## For C 

s.mcitU2 <- with(dataclean, data.frame(pub1=4, female="0_No", phd=4, mcit3=mcit3.U))
head(s.mcitU2)
s.mcitL2 <- with(dataclean, data.frame(pub1=4, female="0_No", phd=4, mcit3=mcit3.L))
head(s.mcitL2)

mcitU.pred2 <- predict(mlogit.mod, s.mcitU2, type="probs", se.fit=T)
mcitL.pred2 <- predict(mlogit.mod, s.mcitL2, type="probs", se.fit=T)
diff.mcit2 <- mcitU.pred2 - mcitL.pred2
diff.mcit2
 
## Other way to Plot Predicted Probabilities

## Load library to reshape long-wide
library(reshape2)
 
## create dataframe
mnlplot <- with(dataclean, data.frame(female = rep(c("1_Yes", "0_No"), each=130), pub1=4, phd=4, mcit3= rep(c(0:129), 2)))

## Store the predicted probabilities
pp <- cbind(mnlplot, predict(mlogit.mod, newdata = mnlplot, type = "probs", se = TRUE))

## calculate the mean probabilities for each level of gender and mcit3
by(pp[, 5:8], pp$female, colMeans)

lpp <- melt(pp, id.vars = c("female", "mcit3", "pub1", "phd"), value.name = "probability")
head(lpp) # view first few rows

## Plot probability for each outcome by gender and citations
ggplot(lpp, aes(x = mcit3, y = probability, colour = female)) +
  geom_line() +
  facet_grid(variable ~ ., scales="free")
 
