
# title: "Section 6: Models for Count Outcomes"
# subtitle: ICPSR CDA
# author: Tamara van der Does\ 2017-07-05

################################################################################

# 1.a Set Up
 
## Load libraries
library(car)
library(foreign)
library(psych)
library(easyGgplot2)
library(aod)
library(MASS)
library(effects)
library(ordinal)
library(reshape2)
library(nnet)

# 1.b Load the Data

data <-as.data.frame(read.dta('https://shawnana79.github.io/data/cda_scireview3.dta', convert.factor=TRUE))
 
# 1.c Examine Data, Keep Variables, Drop Missing, and Verify

head(data)
myvars <- c("female", "enrol", "pub6", "phd") 
datasub <- data[myvars]
datasub$female <- as.factor(datasub$female)
datasub$phd <- as.numeric(datasub$phd)
datasub$enrol <- as.numeric(datasub$enrol)
datasub$pub6 <- as.numeric(datasub$pub6)
dataclean <- na.omit(datasub)
sum(is.na(dataclean)) 
dataclean[!complete.cases(dataclean),]
summary(dataclean)
describe(dataclean)
 
# 3. Estimate the Poisson and NBreg Regression Models

mod.poi <- glm(pub6~female+phd+enrol, family= "poisson", data=dataclean)
summary(mod.poi)
mod.neg <- glm.nb(pub6~female+phd+enrol, data=dataclean)
summary(mod.neg)
 
## Store values of estimates and ratios
varname <- rbind("female", "phd", "enrol")
prm <- cbind(summary(mod.poi)$coef[2:4,1])
nbrm <- cbind(summary(mod.neg)$coef[2:4,1])
ratio1 <- prm/nbrm
prmz <- cbind(summary(mod.poi)$coef[2:4,3])
nbrmz <- cbind(summary(mod.neg)$coef[2:4,3])
ratio2 <- prmz/nbrmz

## Bind, round, and add column names
poinbreg <- cbind(varname, round(prm, digits=3), round(nbrm, digits=3), round(ratio1, digits=3), round(prmz, digits=3), round(nbrmz, digits=3), round(ratio2, digits=3))
rownames(poinbreg) <- NULL
coltitles <-  cbind("Name", "PRM", "NBRM", "Ratio coef", "PRM Z-score", "NBRM Z-score", "Ratio Z-scores")
poinbreg

# 7. Testing NBRM vs. the PRM

## Use LR test to compare models 
 
X2 <- 2 * (logLik(mod.neg) - logLik(mod.poi))
X2
pchisq(X2, df = 1, lower.tail=FALSE)

## Alternatively: Overdispersion tests
library(AER)
dispersiontest(mod.poi, trafo=0)

# 8. Factor changes

bHat<- cbind(coef(mod.neg)[2:4])
Z <- cbind(summary(mod.neg)$coefficients[2:4,3])
Pz <- cbind(summary(mod.neg)$coefficients[2:4,4])
sdx <- rbind(NA, sd(dataclean$phd), sd(dataclean$enrol))

facodds <- cbind(round(bHat, digits=4), round(Z, digits=4), round(Pz, digits=4), round(exp(bHat), digits=4), round(exp(bHat*sdx), digits=4), round(sdx, digits=4))

colnames(facodds) <- cbind("b", "z", "P>|z|", "e^b", "e^bStdX", "StdX")
facodds
 
# 10 Calculating Discrete Channge
  
## Discrete change for B 
s.fem <- with(dataclean, data.frame(female=factor(1:2, levels =1:2, labels = levels(dataclean$female)), phd=mean(phd), enrol=mean(enrol)))
predsfem <- predict(mod.neg, s.fem, type="response", se.fit=TRUE)
diff.fem <- predsfem$fit[2]-predsfem$fit[1]
diff.fem

## Discrete change for C 
phd.U <- mean(dataclean$phd)+(0.5*sd(dataclean$phd))
phd.L <- mean(dataclean$phd)-(0.5*sd(dataclean$phd))

s.phdU <- with(dataclean, data.frame(female="0_No", phd=phd.U, enrol=mean(enrol)))
head(s.phdU)
s.phdL <- with(dataclean, data.frame(female="0_No", phd=phd.L, enrol=mean(enrol)))
head(s.phdL)

phdU.pred <- predict(mod.neg, s.phdU, type="response", se.fit=T)
phdL.pred <- predict(mod.neg, s.phdL, type="response", se.fit=T)
diff.phd <- phdU.pred$fit - phdL.pred$fit
diff.phd
 
# 11. Add Confidence Intervals 

## For B
CIfem.U <- diff.fem + (1.96 * sqrt(predsfem$se.fit[2]^2+predsfem$se.fit[1]^2))
CIfem.L <- diff.fem - (1.96 * sqrt(predsfem$se.fit[2]^2+predsfem$se.fit[1]^2))
cbind(diff.fem, CIfem.L,CIfem.U)

## For C 
CIphd.U <- diff.phd + (1.96 * sqrt((phdU.pred$se.fit)^2+(phdL.pred$se.fit)^2))
CIphd.L <- diff.phd - (1.96 * sqrt((phdU.pred$se.fit)^2+(phdL.pred$se.fit)^2))
cbind(diff.phd, CIphd.L,CIphd.U)
 

# 12. ZIP and ZINP Models 
 
## Load libraries
library(pscl)
  
## ZIP
mod.zip <- zeroinfl(pub6 ~ female+phd+enrol | phd, data = dataclean)
summary(mod.zip)

## ZINP
mod.zinp <- zeroinfl(pub6 ~ female+phd+enrol | phd, data=dataclean, dist="negbin", EM=TRUE)
summary(mod.zinp)
 

# 14. Factor Change for ZINB 
  
## Factor change for those not always zero
bHat<- cbind(coef(mod.zinp)[2:4])
Z <- cbind(summary(mod.zinp)$coefficients$count[2:4,3])
Pz <- cbind(summary(mod.zinp)$coefficients$count[2:4,4])
sdx <- rbind(NA, sd(dataclean$phd), sd(dataclean$enrol))
facodds <- cbind(round(bHat, digits=4), round(Z, digits=4), round(Pz, digits=4), round(exp(bHat), digits=4), round(exp(bHat*sdx), digits=4), round(sdx, digits=4))
colnames(facodds) <- cbind("b", "z", "P>|z|", "e^b", "e^bStdX", "StdX")
facodds

## Factor change in odds of always being zero
bHat.0<- cbind(coef(mod.zinp)[6])
Z.0 <- cbind(summary(mod.zinp)$coefficients$zero[2,3])
Pz.0 <- cbind(summary(mod.zinp)$coefficients$zero[2,4])
sdx.0 <- rbind(sd(dataclean$phd))
facodds.0 <- cbind(round(bHat.0, digits=4), round(Z.0, digits=4), round(Pz.0, digits=4), round(exp(bHat.0), digits=4), round(exp(bHat.0*sdx.0), digits=4), round(sdx.0 , digits=4))
colnames(facodds.0) <- cbind("b", "z", "P>|z|", "e^b", "e^bStdX", "StdX")
facodds.0
 
## 15 Plot Pr(y=0) across the range of C 

s.plot <- data.frame(enrol=mean(dataclean$enrol),
                     phd = rep(seq(from = min(dataclean$phd), to = max(dataclean$phd), length.out= 100), 2),
                     female = "0_No")

## Attach all predictions to subsample
s.plotall1 <- cbind(s.plot, predict(mod.poi, newdata=s.plot, type="link"), rep("Poisson", 100)) 
colnames(s.plotall1) <- c("enrol", "phd", "female", "predict", "method")
s.plotall2 <- cbind(s.plot, predict(mod.neg, newdata=s.plot, type="link"), rep("NBreg", 100)) 
colnames(s.plotall2) <- c("enrol", "phd", "female", "predict", "method")
s.plotall3 <- cbind(s.plot, predict(mod.zip, newdata=s.plot), rep("ZIP", 100)) 
colnames(s.plotall3) <- c("enrol", "phd", "female", "predict", "method")
s.plotall4 <- cbind(s.plot, predict(mod.zinp, newdata=s.plot), rep("ZINP", 100)) 
colnames(s.plotall4) <- c("enrol", "phd", "female", "predict", "method")
s.plot.all <- rbind(s.plotall1, s.plotall2, s.plotall3, s.plotall4)

ggplot(s.plot.all, aes(phd, predict)) + 
  geom_line(aes(colour = method)) +
  labs(x = "phd Score", y = "Predicted Publications")

# 16. Compare Models

## Vuong comparing ZINP and ZIP to non inflated
vuong(mod.poi, mod.zip)
vuong(mod.neg, mod.zinp)
 
